--[[
Restoration Shaman Stat Analyzer, by Rheeah

Expanded and enhanced version based on the original "Resto Shaman Mastery Analyzer" addon by _swift
See:
http://www.curse.com/addons/wow/rshamanmastery

--]]

local revision = tonumber(string.sub("$Revision: 50 $", 12, -3))
_G.RShamanStat = {
    ["revision"] = revision
}
local rss = _G.RShamanStat

-- table to store unrecognized heals for debugging
rss.unknownSpells = { }
-- table to store data for the fight in progress
rss.fight = nil;

-- initialization
function RShamanStatOnLoad(self)
	local lClass, eClass = UnitClass("player");
	if eClass ~= "SHAMAN" then
		return;
	end
	-- enable escape to close the window
	tinsert(UISpecialFrames, "RShamanStatFrame")
		
	-- listen for combat log and other events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("UNIT_MAXHEALTH");
	self:RegisterEvent("UNIT_HEALTH_FREQUENT");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("PLAYER_FOCUS_CHANGED");
	self:RegisterEvent("MASTERY_UPDATE");
	self:RegisterEvent("UNIT_SPELL_HASTE");
	self:RegisterEvent("PLAYER_DAMAGE_DONE_MODS");
	self:RegisterEvent("COMBAT_RATING_UPDATE");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("UNIT_STATS");
	self:RegisterEvent("VARIABLES_LOADED");
	print(("Resto Shaman Stat Analyzer (r%s) loaded!"):format(rss.revision));
end

function RShamanStatOnUpdate(self, elapsed)
	local playerState = rss.playerState;
	if not playerState then return; end
	playerState.busyElapsed = playerState.busyElapsed + elapsed;
	while playerState.busyElapsed > playerState.busyInterval do
		rss.UpdateRing();
		playerState.busyElapsed = playerState.busyElapsed - playerState.busyInterval
	end
end
-- create a frame to catch OnUpdate events for integrator
local f = CreateFrame("frame");
f:SetScript("OnUpdate", RShamanStatOnUpdate);

-- erase current combat data AND all archived/saved data
function rss.ClearAll()
	rss.unknownSpells = { };
    rss.Clear();
    RShamanStatArchive = rss.CreateArchive();
end

-- erase current combat data; called when entering or leaving combat
function rss.Clear()
	rss.ResetPlayerState();
    rss.fight = rss.CreateRecord();
	rss.MonitorState(rss.fight);
end

-- Print data from combat that just ended
-- Save it to archive table
-- Zero accumulators for next combat
function rss.Reset()
	if rss.fight.effectiveTotal > 0 then
		if rss.playerState and rss.playerState.ascendance then
			rss.ProcessCopyRecord(rss.playerState.ascendance, rss.fight);
		end
		if rss.playerState and rss.playerState.ancestralGuidance then
			rss.ProcessCopyRecord(rss.playerState.ancestralGuidance, rss.fight);
		end
        rss.SaveRecord(RShamanStatArchive, rss.fight)
        rss.PrintRecord(rss.fight)
    end
	rss.Clear()
end

SLASH_RSHAMANSTAT1 = '/rss';

function rss.PrintDifficulty()
    if (RShamanStatArchive.filter.difficulty ~= 0) then
        local difficultyName = GetDifficultyInfo(RShamanStatArchive.filter.difficulty);
        print("Instance difficulty is set to '"..difficultyName.."'");
    else
        print("Instance difficulty is set to 'Any'");
    end
end

function rss.PrintUseAll()
    if (RShamanStatArchive.filter.useAll) then
        print("Recording all encounters.");
    else
        print("Recording boss encounters only.");
    end
end

-- Supported commands:
function RShamanStatCmd(msg, ...)
    local msg, args = msg:match("^(%S*)%s*(.-)$");
    if (msg == "help") then
        print(("Resto Shaman Stats (r%s) - Available Commands:"):format(revision));
        print(" * /rss sum          Print all data, and grand totals");
        print(" * /rss sum <boss>   Print data for the specified boss");
        print(" * /rss clr            Print all data, and grand totals, then wipe all data");
        print(" * /rss boss          Only include boss encounter data for grand totals");
        print(" * /rss all            Include all data for grand totals");
        print(" * /rss rsm            Print data of last encounter in RSM mode");
        print(" * /rss difficulty <Any,10N,10H,25H,25N,LFR> Print data from a specific difficulty only");

        rss.PrintUseAll()
        rss.PrintDifficulty()
        return;
    end
	
	if (msg == "") then
		RShamanStatFrame:Show()
	end

    if (msg == "rsm") then
        rss.PrintRSM();
        return;
    end
	
	if msg == "debug" then
		rss.PrintBreakpoints();
		local found;
		for id, spell in pairs(rss.unknownSpells) do
			found = true;
			print("Unrecognized: "..spell.name.."("..id..") : "..spell.count);
		end
		if not found then
			print("No unrecognized spells encountered!");
		end
		
        local filter = function(r)
            return (RShamanStatArchive.filter.useAll or r.isBoss) and (RShamanStatArchive.filter.difficulty == 0 or RShamanStatArchive.filter.difficulty == r.difficulty);
        end
        local summary = rss.CreateAggregateRecord("Summary", RShamanStatArchive, filter);
		if (summary.effectiveTotal > 0) then
			for reason, leak in pairs(summary.leakage) do
				print(("%s : %.1f  (%.2f%%)"):format(reason, leak, 100*leak/summary.effectiveTotal));
			end
		end
	end

    if (msg == "sum" and args ~= "") then
        local r = rss.CreateAggregateRecord(args, RShamanStatArchive, function(r)
            return (r.encounterName == args) and (RShamanStatArchive.filter.difficulty == 0 or RShamanStatArchive.filter.difficulty == r.difficulty);
        end)
        rss.args = args
        rss.PrintRecord(r)
        return;
    end

    if (msg == "difficulty" and args ~= "") then
        if args == "10N" then
            RShamanStatArchive.filter.difficulty = 3;
        elseif args == "25N" then
            RShamanStatArchive.filter.difficulty = 4;
        elseif args == "10H" then
            RShamanStatArchive.filter.difficulty = 5;
        elseif args == "25H" then
            RShamanStatArchive.filter.difficulty = 6;
        elseif args == "LFR" then
            RShamanStatArchive.filter.difficulty = 7;
        else
            RShamanStatArchive.filter.difficulty = 0;
        end
        rss.PrintDifficulty()
        return;
    end

    if (msg == "boss") then
        RShamanStatArchive.filter.useAll = false;
        rss.PrintUseAll()
        return
	elseif (msg == "all") then
        RShamanStatArchive.filter.useAll = true;
        rss.PrintUseAll()
        return
    end

	if (msg == "sum" or msg == "clr") then
        local filter = function(r)
            return (RShamanStatArchive.filter.useAll or r.isBoss) and (RShamanStatArchive.filter.difficulty == 0 or RShamanStatArchive.filter.difficulty == r.difficulty)
        end
		if (#RShamanStatArchive > 0) then
            for i, v in ipairs(RShamanStatArchive) do
                if (filter(v)) then
                    rss.PrintRecord(v)
                end
			end
        end

        local summary = rss.CreateAggregateRecord("Summary", RShamanStatArchive, filter)
        rss.PrintRecord(summary)
		print("---------------");
	end
	if (msg == "clr") then
		rss.ClearAll();
		print("RSS:: All archived data cleared");
	end
end
SlashCmdList["RSHAMANSTAT"] = RShamanStatCmd;

	
function RShamanStatOnEvent(self, event, ...)
	local currentSpec = GetSpecialization()
	local role = currentSpec and select(6, GetSpecializationInfo(currentSpec)) or "NONE"

    if event == "VARIABLES_LOADED" then
		rss.Clear();
        RShamanStatArchive = rss.LoadArchive(RShamanStatArchive)
        if RShamanStatArchive and #RShamanStatArchive > 0 then
            DEFAULT_CHAT_FRAME:AddMessage("RSS: "..#RShamanStatArchive.." archived records found.");
        end
		rss.UpdateUI();
        return;
    end
	if role ~= "HEALER" then
		return;
    end

	if event == "UNIT_HEALTH_FREQUENT" or event == "UNIT_MAXHEALTH" then
		rss.UpdateHealthTable(...);
		return;
	end

	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		rss.CombatLogEntry(...);
		return;
	end	
	
	if event == "PLAYER_TARGET_CHANGED" then
		rss.UpdateUnitHealth("target");
		return;
	end
	
	if event == "PLAYER_FOCUS_CHANGED" then
		rss.UpdateUnitHealth("focus");
		return;
	end
	
	if event == "MASTERY_UPDATE" or event == "PLAYER_DAMAGE_DONE_MODS" or event == "COMBAT_RATING_UPDATE" or 
	   (event == "UNIT_SPELL_HASTE" and UnitIsUnit(select(1,...),"player")) or
	   (event == "UNIT_AURA" and UnitIsUnit(select(1,...),"player")) or 
	   (event == "UNIT_STATS" and UnitIsUnit(select(1,...),"player")) then
		rss.MonitorState(rss.fight);
	end
	
	if event == "PLAYER_REGEN_DISABLED" then
		-- entering combat; reset counters
		rss.Clear();
		return;
	end
	
	if event == "PLAYER_REGEN_ENABLED" then
		-- leaving combat: print status and reset counters
		rss.Reset();
		return;
	end
end
